#!/usr/bin/python

#
#  sobexsrv (c) Collin R. Mulliner <collin-sobexsrv(AT)betaversion.net>
#               web: http://www.mulliner.org/bluetooth/
#
#  license: GPL
#

import os, sys, string

# *** CONFIGURATION ***

# --- log level ---
# 0 no logging
# 1 transfer log
# 2 debug
LOGLEVEL = 1
LOGFILE = "/tmp/sobexsrv.log"

# --- basepath ---
# this is the default INBOX
BASEPATH = "/tmp/"

# --- OVERWRITE ---
# can files and directorys be overwritten or deleted
# 0 no
# 1 yes
OVERWRITE = 0 

# --- SECURITY LEVEL --- 
# 1 no security (everyone can connect)
# 2 authenticate
# 3 auth + encrypt
SECURITYLEVEL = 1

# *** END CONFIG ***

# --- HELPERS ---

def log_close():
 logfd.close()

def clean_exit(code):
 log_close()
 sys.exit(code)

def log_open(logfile):
 global logfd
 logfd = open(logfile, "a")
 if logfd == -1:
  print "can't open logfile\n"
  sys.exit(0)

def log(level, string):
 if level <= LOGLEVEL:
  logfd.write("[" + remote + " " + connectionid + "] " + string)

# --- HANDLER STARTS HERE ---

# start logging
log_open(LOGFILE)

# -- READ header --
line = "just dot have an empty line"
path = ""
while len(line) > 2:
 line = sys.stdin.readline()
 if len(line) < 2:
  break
 log(2, "header_line: " + line)
 line = string.rstrip(line, "\n")
 tmp = line.split(None, 1)
 if tmp[0].find("command:") == 0:
  command = tmp[1]
 if tmp[0].find("length:") == 0:
  length = tmp[1]
 if tmp[0].find("path:") == 0:
  path = tmp[1]
 if tmp[0].find("type:") == 0:
  type = tmp[1]
 if tmp[0].find("local:") == 0:
  local = tmp[1]
 if tmp[0].find("remote:") == 0:
  remote = tmp[1]
 if tmp[0].find("filename:") == 0:
  filename = tmp[1]
 if tmp[0].find("description:") == 0:
  description = tmp[1]
 if tmp[0].find("connectionid:") == 0:
  connectionid = tmp[1]
 if tmp[0].find("count:") == 0:
  count = tmp[1]
 if tmp[0].find("datatype:") == 0:
  datatype = tmp[1]

# -- PROCESS commands --

# -- PUT --
if command == "put":
 fullpath = BASEPATH + path + "/" + filename
 log(1, "PUT " + fullpath + " " + length + " Bytes\n")
 if len(filename) == 1:
  clean_exit(2)
 # create
 if datatype == 1:
  if os.access(fullpath, os.F_OK) and OVERWRITE == 0:
   log(1, "PUT " + fullpath + " exits and overwrite is disabled\n")
   clean_exit(2)
  fp = open(fullpath, "w")
  if fp == -1:
   log(1, "PUT " + fullpath + " can't be created/overwritten\n")
   clean_exit(2)
  buf = sys.stdin.read(int(length))
  fp.write(buf)
  fp.close()
  log(1, "PUT " + fullpath + " saved\n")
  clean_exit(1)
 # delete file/directory
 else:
  if not os.access(fullpath, os.F_OK) or OVERWRITE == 0:
   log(1, "PUT (delete) " + fullpath + " no such file or no permissions\n")
   clean_exit(2)
  else:
   try:
    os.unlink(fullpath)
    log(1, "PUT (delete)" + fullpath + " deleted\n")
   except OSError:
    os.rmdir(fullpath)
    log(1, "PUT (delete)" + fullpath + " deleted\n")
   clean_exit(1)

# -- GET --
elif command == "get":
 # folder listing
 if type == "x-obex/folder-listing":
  log(1, "GET listing of folder: " + BASEPATH + path + "\n")
  if os.access(BASEPATH + path, os.F_OK) == -1:
   log(1, "GET " + BASEPATH + path + " can't be accessed\n")
  buf = "<?xml version=\"1.0\"?>\n" + "<!DOCTYPE folder-listing SYSTEM \"obex-folder-listing.dtd\">\n" + "<folder-listing version=\"1.0\">\n"
  lsout = os.popen("ls -la " + BASEPATH + path, "r")
  line = "no empty line"
  while len(line) > 0:
   line = lsout.readline()
   line = string.rstrip(line, "\n")
   parts = line.split(None, 8)
   if len(parts) > 3:
    if parts[0].find("d") == 0:
     if parts[8] == "..":
      buf = buf + "<partenfolder/>\n"
     elif parts[8] != ".":
      buf = buf + "<folder name=\"" + parts[8] + "\" size=\"" + parts[4] + "\"/>\n"
    elif parts[0].find("-") == 0:
     buf = buf + "<file name=\"" + parts[8] + "\" size=\"" + parts[4] + "\"/>\n"
  buf = buf + "</folder-listing>"
  print "length: " + str(len(buf)) + "\n"
  log(2, "GET listing length " + str(len(buf)) + "\n")
  print buf
  log(2, "GET listing: " + buf + "\n");
  clean_exit(1)

 # regular get
 fullpath = BASEPATH + path + "/" + filename
 log(1, "GET " + fullpath + "\n")
 fp = open(fullpath)
 if fp == -1:
  log(1, "GET can't open " + fullpath + "\n")
  clean_exit(2)
 fp.seek(0, 2)
 fpsize = fp.tell()
 fp.seek(0, 0)
 buf = fp.read(fpsize)
 print "length: " + str(fpsize) + "\n"
 log(1, "GET " + fullpath + " " + str(fpsize) + " Bytes\n")
 print buf
 fp.close()
 clean_exit(1)

# -- CONNECTION STUFF --
elif command == "connect":
 log(1, "CONNECT\n")
 clean_exit(1)
elif command == "disconnect":
 log(1, "DISCONNECT\n")
 clean_exit(1)
elif command == "accept":
 log(1, "ACCEPT at security level " + str(SECURITYLEVEL) + "\n")
 clean_exit(SECURITYLEVEL)
elif command == "setpath":
 clean_exit(1)
 
# -- MKDIR --
# is not an actual obex command, but its emulated by sobexsrv
elif command == "mkdir":
 log(1, "MKDIR " + BASEPATH + path + "\n")
 mkd = os.popen("mkdir " + BASEPATH + path, "r")
 line = mkd.readline()
 if len(line) > 0:
  log(1, "MKDIR can't create " + BASEPATH + path + "\n")
  clean_exit(2)
 else:
  log(1, "MKDIR " + BASEPATH + path + " created\n")
  clean_exit(1)
