/*
 *  $Id$
 */

/*
 *  sobexsrv (c) Collin R. Mulliner <collin-sobexsrv(AT)betaversion.net>
 *               web: http://www.mulliner.org/bluetooth/
 *
 *  license: GPL
 *
 */

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

/*
 *  all code taken from hcitool (www.bluez.org)
 */
static int find_conn(int s, int dev_id, long arg)
{
	struct hci_conn_list_req *cl;
	struct hci_conn_info *ci;
	int i;

	if (!(cl = malloc(10 * sizeof(*ci) + sizeof(*cl)))) {
		perror("Can't allocate memory");
		exit(-1);
	}
	cl->dev_id = dev_id;
	cl->conn_num = 10;
	ci = cl->conn_info;

	if (ioctl(s, HCIGETCONNLIST, (void *) cl)) {
		perror("Can't get connection list");
		exit(-1);
	}

	for (i = 0; i < cl->conn_num; i++, ci++)
		if (!bacmp((bdaddr_t *) arg, &ci->bdaddr))
			return(1);

	return(0);
}

int cmd_auth(int dev_id, bdaddr_t *bdaddr)
{
	struct hci_conn_info_req *cr = NULL;
	int dd;


	if (dev_id < 0) {
		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) bdaddr);
		if (dev_id < 0) {
			fprintf(stderr, "Not connected.\n");
			return(0);
		}
	}

	dd = hci_open_dev(dev_id);
	if (dd < 0) {
		perror("HCI device open failed");
		return(0);
	}

	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
	if (!cr) {
		perror("Can't allocate memory");
		goto err;
	}

	bacpy(&cr->bdaddr, bdaddr);
	cr->type = ACL_LINK;
	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
		perror("Get connection info failed");
		goto err;
	}

	if (hci_authenticate_link(dd, htobs(cr->conn_info->handle), 25000) < 0) {
		perror("HCI authentication request failed");
		goto err;
	}

	close(dd);
	free(cr);
	return(1);

err:
	close(dd);
	if (cr != NULL) free(cr);
	return(0);
}

int cmd_enc(int dev_id, bdaddr_t *bdaddr)
{
	struct hci_conn_info_req *cr = NULL;
	uint8_t encrypt;
	int dd;

	
	if (dev_id < 0) {
		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) bdaddr);
		if (dev_id < 0) {
			fprintf(stderr, "Not connected.\n");
			return(0);
		}
	}

	dd = hci_open_dev(dev_id);
	if (dd < 0) {
		perror("HCI device open failed");
		return(0);
	}
 
	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
	if (!cr) {
		perror("Can't allocate memory");
		goto err;
	}

	bacpy(&cr->bdaddr, bdaddr);
	cr->type = ACL_LINK;
	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
		perror("Get connection info failed");
		goto err;
	}

	encrypt = 1;

	if (hci_encrypt_link(dd, htobs(cr->conn_info->handle), encrypt, 25000) < 0) {
		perror("HCI set encryption request failed");
		goto err;
	}

	close(dd);
	free(cr);
	return(1);

err:
	close(dd);
	if (cr != NULL) free(cr);
	return(0);
}

/*
 *  $Log$
 */
